@@ -1,11 +1,11 @@
 /*
- * $Id: FrontendConfig.cs 427 2008-09-27 23:50:51Z meebey $
- * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Engine/Config/FrontendConfig.cs $
- * $Rev: 427 $
- * $Author: meebey $
- * $Date: 2008-09-28 01:50:51 +0200 (Sun, 28 Sep 2008) $
+ * $Id$
+ * $URL$
+ * $Rev$
+ * $Author$
+ * $Date$
  *
- * smuxi - Smart MUltipleXed Irc
+ * Smuxi - Smart MUltipleXed Irc
  *
  * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
  *
@@ -97,11 +97,12 @@ namespace Smuxi.Engine
                 LoadEntry(eprefix+"Port", null);
                 LoadEntry(eprefix+"Channel", null);
                 LoadEntry(eprefix+"Formatter", null);
-                LoadEntry(eprefix+"UseSshTunnel", null);
+                LoadEntry(eprefix+"UseSshTunnel", false);
                 LoadEntry(eprefix+"SshProgram", null);
-                LoadEntry(eprefix+"SshHostname", null);
-                LoadEntry(eprefix+"SshPort", null);
-                LoadEntry(eprefix+"SshUsername", null);
+                LoadEntry(eprefix+"SshParameters", null);
+                LoadEntry(eprefix+"SshHostname", String.Empty);
+                LoadEntry(eprefix+"SshPort", 22);
+                LoadEntry(eprefix+"SshUsername", String.Empty);
                 LoadEntry(eprefix+"SshPassword", null);
             }
             
