@@ -93,7 +93,7 @@ namespace Smuxi.Engine
         
         public bool IsAlive {
             get {
-                return  !f_TaskQueue.Disposed;
+                return !f_TaskQueue.Disposed;
             }
         }
         
@@ -146,7 +146,13 @@ namespace Smuxi.Engine
         public void Sync()
         {
             Trace.Call();
-            
+
+            // DESIGN BUG: Something is fishy here, if a chat is created and
+            // added to the session right before we sync it will be in an
+            // unpopulated state. For example no populated person list... how
+            // should we handle this? Only sync ready chats and sync those
+            // later using some kind of delayed queue?!?
+
             // TODO: sort pages network tabs then channel tabs (alphabeticly)
             // sync pages            
             lock (_Session.Chats) {
@@ -210,7 +216,7 @@ namespace Smuxi.Engine
             if (CurrentProtocolManager != null) {
                 SetNetworkStatus(CurrentProtocolManager.ToString());
             } else {
-                SetNetworkStatus(String.Format("({0})", _("no network connections")));
+                SetNetworkStatus(String.Format("({0})", _("No network connections")));
             }
         }
         
@@ -232,7 +238,8 @@ namespace Smuxi.Engine
         {
             AddMessageToChat(chat, new MessageModel(text));
         }
-        
+
+        [Obsolete("This method is unsafe, use AddTextToChat(cmd.Chat, text) instead!", true)]
         public void AddTextToCurrentChat(string text)
         {
             AddTextToChat(CurrentChat, text);
@@ -259,7 +266,8 @@ namespace Smuxi.Engine
         {
             if (!_SyncedChats.Contains(chat)) {
 #if LOG4NET
-                _Logger.Warn("AddMessageToChat(): chat: " + chat + " is not synced yet, ignoring call...");
+                // too much logging noise
+                //_Logger.Warn("AddMessageToChat(): chat: " + chat + " is not synced yet, ignoring call...");
 #endif
                 return;
             }
@@ -274,6 +282,7 @@ namespace Smuxi.Engine
             });
         }
         
+        [Obsolete("This method is unsafe, use AddMessageToChat(cmd.Chat, msg) instead!", true)]
         public void AddMessageToCurrentChat(MessageModel msg)
         {
             AddMessageToChat(CurrentChat, msg);
