@@ -1,13 +1,7 @@
 /*
- * $Id$
- * $URL$
- * $Rev$
- * $Author$
- * $Date$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2010 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -142,24 +136,28 @@ namespace Smuxi.Engine
             // if this is the first frontend, we process OnStartupCommands
             if (!_OnStartupCommandsProcessed) {
                 _OnStartupCommandsProcessed = true;
-                
+
+                string str;
                 MessageModel msg;
                 msg = new MessageModel();
                 msg.MessageParts.Add(
                     new TextMessagePartModel(new TextColor(0xFF0000), null, false,
                             true, false, _("Welcome to Smuxi")));
                 AddMessageToChat(_SessionChat, msg);
-                
+
                 msg = new MessageModel();
                 msg.MessageParts.Add(
                     new TextMessagePartModel(null, null, false,
                             true, false, _("Type /help to get a list of available commands.")));
                 AddMessageToChat(_SessionChat, msg);
 
+                str = _("After you have made a connection the list of " +
+                        "available commands changes. Use the /help command " +
+                        "again to see the extended command list.");
                 msg = new MessageModel();
                 msg.MessageParts.Add(
                     new TextMessagePartModel(null, null, false,
-                            true, false, _("After you have made a connection the list of available commands changes, just use /help again.")));
+                            true, false, str));
                 AddMessageToChat(_SessionChat, msg);
                 
                 foreach (string command in (string[])_UserConfig["OnStartupCommands"]) {
@@ -185,44 +183,33 @@ namespace Smuxi.Engine
                     if (!server.OnStartupConnect) {
                         continue;
                     }
-                    
-                    IProtocolManager protocolManager = _CreateProtocolManager(fm, server.Protocol);
-                    if (protocolManager == null) {
+
+                    bool isError = false;
+                    try {
+                        IProtocolManager protocolManager = Connect(server, fm);
+
+                        // if the connect command was correct, we should be
+                        // able to get the chat model
+                        if (protocolManager.Chat == null) {
+                            isError = true;
+                        }
+                    } catch (Exception ex) {
+#if LOG4NET
+                        f_Logger.Error("RegisterFrontendUI(): Exception during "+
+                                       "automatic connect: ", ex);
+#endif
+                        isError = true;
+                    }
+                    if (isError) {
+                        fm.AddTextToChat(
+                            _SessionChat,
+                            String.Format(
+                                _("Automatic connect to {0} failed!"),
+                                server.Hostname + ":" + server.Port
+                            )
+                        );
                         continue;
                     }
-                    
-                    _ProtocolManagers.Add(protocolManager);
-                    string password = null;
-                    // only pass non-empty passwords to Connect()
-                    if (!String.IsNullOrEmpty(server.Password)) {
-                        password = server.Password;
-                    }
-                    protocolManager.Connect(fm, server.Hostname, server.Port,
-                                            server.Username,
-                                            password);
-                    // if the connect command was correct, we should be able to get
-                    // the chat model
-                    if (protocolManager.Chat == null) {
-                        fm.AddTextToChat(_SessionChat, String.Format(_("Automatic connect to {0} failed!"), server.Hostname + ":" + server.Port));
-                        continue;
-                    }
-                    
-                    if (server.OnConnectCommands != null && server.OnConnectCommands.Count > 0) {
-                        // copy the server variable into the loop scope, else it will always be the same object in the anonymous method!
-                        ServerModel ser = server;
-                        protocolManager.Connected += delegate {
-                            foreach (string command in ser.OnConnectCommands) {
-                                if (command.Length == 0) {
-                                    continue;
-                                }
-                                CommandModel cd = new CommandModel(fm,
-                                                                   protocolManager.Chat,
-                                                                   (string)_UserConfig["Interface/Entry/CommandCharacter"],
-                                                                   command);
-                                protocolManager.Command(cd);
-                            }
-                        };
-                    }                                   
                 }
             }
         }
@@ -250,13 +237,17 @@ namespace Smuxi.Engine
                                "frontend manager (probably already cleanly " +
                                " deregistered), ignoring...");
 #endif
-                //throw new InvalidOperationException("Could not find key for frontend manager in _FrontendManagers.");
                 return;
             }
             
             lock (_FrontendManagers) {
                 _FrontendManagers.Remove(key);
             }
+
+#if LOG4NET
+            f_Logger.Debug("DeregisterFrontendUI(fm): disposing FrontendManager");
+#endif
+            fm.Dispose();
         }
         
         public void DeregisterFrontendUI(IFrontendUI ui)
@@ -269,11 +260,23 @@ namespace Smuxi.Engine
             
             string uri = GetUri(ui);
 #if LOG4NET
-            f_Logger.Debug("Deregistering UI with URI: "+uri);
+            f_Logger.Debug("DeregisterFrontendUI(ui): deregistering UI with URI: "+uri);
 #endif
+            FrontendManager manager;
             lock (_FrontendManagers) {
+                _FrontendManagers.TryGetValue(uri, out manager);
                 _FrontendManagers.Remove(uri);
             }
+            if (manager == null) {
+#if LOG4NET
+                f_Logger.Error("DeregisterFrontendUI(ui): can't dispose as FrontendManager not found with URI: " + uri);
+#endif
+            } else {
+#if LOG4NET
+                f_Logger.Debug("DeregisterFrontendUI(ui): disposing FrontendManager with URI: " + uri);
+#endif
+                manager.Dispose();
+            }
         }
         
         public FrontendManager GetFrontendManager(IFrontendUI ui)
@@ -389,12 +392,14 @@ namespace Smuxi.Engine
             TextMessagePartModel msgPart;
             
             msgPart = new TextMessagePartModel();
-            msgPart.Text = _("[Engine Commands]");
+            // TRANSLATOR: this line is used as a label / category for a
+            // list of commands below
+            msgPart.Text = "[" + _("Engine Commands") + "]";
             msgPart.Bold = true;
             msg.MessageParts.Add(msgPart);
-            
+
             cd.FrontendManager.AddMessageToChat(cd.Chat, msg);
-            
+
             string[] help = {
                 "help",
                 "connect/server protocol [protocol-parameters]",
@@ -406,7 +411,7 @@ namespace Smuxi.Engine
             };
             
             foreach (string line in help) { 
-                cd.FrontendManager.AddTextToCurrentChat("-!- " + line);
+                cd.FrontendManager.AddTextToChat(cd.Chat, "-!- " + line);
             }
         }
         
@@ -448,21 +453,34 @@ namespace Smuxi.Engine
             */
 
             if (protocolManager == null) {
-                protocolManager = _CreateProtocolManager(fm, protocol);
-                if (protocolManager == null) {
+                try {
+                    protocolManager = CreateProtocolManager(protocol);
+                    _ProtocolManagers.Add(protocolManager);
+                } catch (ArgumentException ex) {
+                    if (ex.ParamName != "protocol") {
+                        throw;
+                    }
+                    // this is an unknown protocol error
+                    fm.AddTextToChat(
+                        fm.CurrentChat,
+                        String.Format("-!- {0}", ex.Message)
+                    );
                     return;
                 }
-                _ProtocolManagers.Add(protocolManager);
             }
             // HACK: this is hacky as the Command parser of the protocol manager
             // will pass this command to it's connect method only if cd was
             // constructed correctly beginning with /connect
             // So make sure it's like it needs to be!
             if (cd.Command != "connect") {
-                throw new ArgumentException("cd.Command must be 'connect' but was: '" + cd.Command + "'.", "cd");
+                string cmd = String.Format("{0}connect {1}",
+                                cd.CommandCharacter,
+                                String.Join(" ", cd.DataArray, 1,
+                                            cd.DataArray.Length - 1));
+                cd = new CommandModel(fm, cd.Chat, cd.CommandCharacter, cmd);
             }
             protocolManager.Command(cd);
-            
+
             // set this as current protocol manager
             // but only if there was none set (we might be on a chat for example)
             // or if this is the neutral "smuxi" tab
@@ -493,10 +511,11 @@ namespace Smuxi.Engine
                         }
                     }
                 }
-                fm.AddTextToCurrentChat(
+                fm.AddTextToChat(
+                    cd.Chat,
                     "-!- " +
                     String.Format(
-                        _("Disconnect failed, could not find server: {0}"),
+                        _("Disconnect failed - could not find server: {0}"),
                         server
                     )
                 );
@@ -531,17 +550,17 @@ namespace Smuxi.Engine
                 switch (cd.DataArray[1].ToLower()) {
                     case "load":
                         _Config.Load();
-                        fm.AddTextToCurrentChat("-!- " +
+                        fm.AddTextToChat(cd.Chat, "-!- " +
                             _("Configuration reloaded"));
                         break;
                     case "save":
                         _Config.Save();
-                        fm.AddTextToCurrentChat("-!- " +
+                        fm.AddTextToChat(cd.Chat, "-!- " +
                             _("Configuration saved"));
                         break;
                     default:
-                        fm.AddTextToCurrentChat("-!- " + 
-                            _("Invalid paramater for config, use load or save"));
+                        fm.AddTextToChat(cd.Chat, "-!- " + 
+                            _("Invalid parameter for config; use load or save"));
                         break;
                 }
             } else {
@@ -570,8 +589,8 @@ namespace Smuxi.Engine
                         _CommandNetworkClose(cd);
                         break;
                     default:
-                        fm.AddTextToCurrentChat("-!- " + 
-                            _("Invalid paramater for network, use list, switch or close"));
+                        fm.AddTextToChat(cd.Chat, "-!- " + 
+                            _("Invalid parameter for network; use list, switch, or close"));
                         break;
                 }
             } else {
@@ -582,10 +601,10 @@ namespace Smuxi.Engine
         private void _CommandNetworkList(CommandModel cd)
         {
             FrontendManager fm = cd.FrontendManager;
-            fm.AddTextToCurrentChat("-!- " + _("Networks") + ":");
+            fm.AddTextToChat(cd.Chat, "-!- " + _("Networks") + ":");
             lock (_ProtocolManagers) {
                 foreach (IProtocolManager nm in _ProtocolManagers) {
-                    fm.AddTextToCurrentChat("-!- " +
+                    fm.AddTextToChat(cd.Chat, "-!- " +
                         _("Type") + ": " + nm.Protocol + " " +
                         _("Host") + ": " + nm.Host + " " + 
                         _("Port") + ": " + nm.Port);
@@ -609,8 +628,8 @@ namespace Smuxi.Engine
                     }
                 }
                 if (pm == null) {
-                    fm.AddTextToCurrentChat("-!- " +
-                        String.Format(_("Network close failed, could not find network with host: {0}"),
+                    fm.AddTextToChat(cd.Chat, "-!- " +
+                        String.Format(_("Network close failed - could not find network with host: {0}"),
                                       host));
                     return;
                 }
@@ -643,8 +662,8 @@ namespace Smuxi.Engine
                         }
                     }
                 }
-                fm.AddTextToCurrentChat("-!- " +
-                    String.Format(_("Network switch failed, could not find network with host: {0}"),
+                fm.AddTextToChat(cd.Chat, "-!- " +
+                    String.Format(_("Network switch failed - could not find network with host: {0}"),
                                   host));
             } else if (cd.DataArray.Length >= 2) {
                 // next network manager
@@ -656,13 +675,25 @@ namespace Smuxi.Engine
         
         private void _NotConnected(CommandModel cd)
         {
-            cd.FrontendManager.AddTextToCurrentChat("-!- " + _("Not connected to any network"));
+            cd.FrontendManager.AddTextToChat(
+                cd.Chat,
+                String.Format("-!- {0}",
+                    _("Not connected to any network")
+                )
+            );
         }
         
         private void _NotEnoughParameters(CommandModel cd)
         {
-            cd.FrontendManager.AddTextToCurrentChat("-!- " +
-                String.Format(_("Not enough parameters for {0} command"), cd.Command));
+            cd.FrontendManager.AddTextToChat(
+                cd.Chat,
+                String.Format("-!- {0}",
+                    String.Format(
+                        _("Not enough parameters for {0} command"),
+                        cd.Command
+                    )
+                )
+            );
         }
         
         public void UpdateNetworkStatus()
@@ -923,42 +954,78 @@ namespace Smuxi.Engine
             return _ProtocolManagerFactory.GetProtocols();
         }
         
-        public void Connect(FrontendManager frontendManager, string protocol,
-                            string hostname, int port,
-                            string username, string password)
+        public IProtocolManager Connect(ServerModel server, FrontendManager frontendManager)
         {
-            Trace.Call(frontendManager, protocol, hostname, port, username, "XXX");
+            Trace.Call(server, frontendManager);
             
-            IProtocolManager protocolManager = _CreateProtocolManager(
-                frontendManager,
-                protocol
+            if (server == null) {
+                throw new ArgumentNullException("server");
+            }
+            if (String.IsNullOrEmpty(server.Protocol)) {
+                throw new ArgumentNullException("server.Protocol");
+            }
+            if (frontendManager == null) {
+                throw new ArgumentNullException("frontendManager");
+            }
+
+            IProtocolManager protocolManager = CreateProtocolManager(
+                server.Protocol
             );
-            if (protocolManager == null) {
-                throw new ApplicationException(
-                    String.Format(
-                        _("No protocol manager found for the protocol: {0}"),
-                        protocol
-                    )
-                );
-            }
-            
             _ProtocolManagers.Add(protocolManager);
+
+            string password = null;
             // only pass non-empty passwords to Connect()
-            if (String.IsNullOrEmpty(password)) {
-                password = null;
+            if (!String.IsNullOrEmpty(server.Password)) {
+                password = server.Password;
             }
-            protocolManager.Connect(frontendManager, hostname, port,
-                                    username, password);
+            protocolManager.Connect(frontendManager, server.Hostname,
+                                    server.Port, server.Username, password);
+            if (protocolManager.Chat == null) {
+                // just in case the ProtocolManager is not setting the
+                // protocol chat
+                throw new ApplicationException(_("Connect failed."));
+            }
+
+            if (server.OnConnectCommands != null && server.OnConnectCommands.Count > 0) {
+                protocolManager.Connected += delegate {
+                    foreach (string command in server.OnConnectCommands) {
+                        if (command.Length == 0) {
+                            continue;
+                        }
+                        CommandModel cd = new CommandModel(
+                            frontendManager,
+                            protocolManager.Chat,
+                            (string) _UserConfig["Interface/Entry/CommandCharacter"],
+                            command
+                        );
+                        protocolManager.Command(cd);
+                    }
+                };
+            }
+
+            return protocolManager;
         }
         
-        private IProtocolManager _CreateProtocolManager(FrontendManager fm, string protocol)
+        private IProtocolManager CreateProtocolManager(string protocol)
         {
-            ProtocolManagerInfoModel info = _ProtocolManagerFactory.GetProtocolManagerInfoByAlias(protocol); 
+            ProtocolManagerInfoModel info =
+                _ProtocolManagerFactory.GetProtocolManagerInfoByAlias(protocol);
             if (info == null) {
-                fm.AddTextToCurrentChat("-!- " + String.Format(
-                        _("Unknown protocol: {0}"), protocol));
-                return null;
+                if (_ProtocolManagerFactory.ProtocolManagerInfos.Count != 1) {
+                    throw new ArgumentException(
+                            String.Format(
+                                _("No protocol manager found for the protocol: {0}"),
+                                protocol
+                            ),
+                            "protocol"
+                    );
+                }
+
+                // ok, we forgive the user not passing a valid protocol by
+                // falling back to the only available protocol
+                info = _ProtocolManagerFactory.ProtocolManagerInfos[0];
             }
+
             return _ProtocolManagerFactory.CreateProtocolManager(info, this);
         }
         
