@@ -45,6 +45,7 @@ namespace Smuxi.Frontend.Gnome
         private int              _HistoryPosition;
         private bool             _HistoryChangedLine;
         private Notebook         _Notebook;
+        private CommandManager   _CommandManager;
         
         /*
         public StringCollection History {
@@ -78,10 +79,21 @@ namespace Smuxi.Frontend.Gnome
 
         public Entry(Notebook notebook)
         {
+            Trace.Call(notebook);
+
+            if (notebook == null) {
+                throw new ArgumentNullException("notebook");
+            }
+
             _History.Add(String.Empty);
             
             _Notebook = notebook;
-            
+
+            InitCommandManager();
+            Frontend.SessionPropertyChanged += delegate {
+                InitCommandManager();
+            };
+
             Activated += new EventHandler(_OnActivated);
             KeyPressEvent += new Gtk.KeyPressEventHandler(_OnKeyPress);
             FocusOutEvent += new Gtk.FocusOutEventHandler(_OnFocusOut);
@@ -186,11 +198,15 @@ namespace Smuxi.Frontend.Gnome
         [GLib.ConnectBefore]
         private void _OnKeyPress(object sender, Gtk.KeyPressEventArgs e)
         {
-            Trace.Call(sender, e);
-            
+            // too much logging noise
+            //Trace.Call(sender, e);
+
 #if LOG4NET
-            _Logger.Debug("_OnKeyPress(): Key: " + e.Event.Key.ToString() + 
+            // too much logging noise
+            /*
+            _Logger.Debug("_OnKeyPress(): Key: " + e.Event.Key.ToString() +
                           " KeyValue: " + e.Event.KeyValue);
+            */
 #endif
 
             int keynumber = (int)e.Event.KeyValue;
@@ -296,16 +312,14 @@ namespace Smuxi.Frontend.Gnome
             UpdateHistoryChangedLine();
             switch (key) {
                 case Gdk.Key.Tab:
+                    // don't let GTK handle the focus, as we will do it
                     e.RetVal = true;
                     if (Frontend.MainWindow.CaretMode) {
                         // when we are in caret-mode change focus to output textview
                         _Notebook.CurrentChatView.HasFocus = true;
                     } else {
-                        // don't loose the focus (if we are not in caret-mode)
-                        if (_Notebook.CurrentChatView.ChatModel is GroupChatModel) {
-                            if (Text.Length > 0) {
-                                _NickCompletion();
-                            }
+                        if (Text.Length > 0) {
+                            _NickCompletion();
                         }
                     }
                     break;
@@ -345,10 +359,14 @@ namespace Smuxi.Frontend.Gnome
             GLib.Timeout.Add(250, new GLib.TimeoutHandler(delegate {
                 // TODO: check mouse buttons, if left mouse button is still pressed
                 // we should not interrupt either, as the user is going to make a selection!
-                
+
+                ChatView chat = Frontend.MainWindow.Notebook.CurrentChatView;
+                if (chat == null) {
+                    return false;
+                }
+
                 // don't interrupt on-going selections
-                if (Frontend.MainWindow.Notebook.CurrentChatView.HasSelection &&
-                    Frontend.MainWindow.Notebook.CurrentChatView.HasFocus) {
+                if (chat.HasSelection && chat.HasFocus) {
 #if LOG4NET
                     //_Logger.Debug("_OnFocusOut(): CurrentChatView has on-going selection, waiting..."); 
 #endif
@@ -382,7 +400,7 @@ namespace Smuxi.Frontend.Gnome
                     // seems to be a paste, so let's break it apart
                     string[] msgParts = Text.Split(new char[] {'\n'});
                     if (msgParts.Length > 3) {
-                        string msg = String.Format(_("You are going to paste {0} lines, do you want to continue?"),
+                        string msg = String.Format(_("You are going to paste {0} lines. Do you want to continue?"),
                                                    msgParts.Length);
                         Gtk.MessageDialog md = new Gtk.MessageDialog(
                                                     Frontend.MainWindow,
@@ -425,27 +443,19 @@ namespace Smuxi.Frontend.Gnome
             if (!(cmd.Length > 0)) {
                 return;
             }
-            
-            bool handled;
-            CommandModel cd = new CommandModel(Frontend.FrontendManager, _Notebook.CurrentChatView.ChatModel,
-                                    (string)Frontend.UserConfig["Interface/Entry/CommandCharacter"],
-                                    cmd);
-            handled = _Command(cd);
-            if (!handled) {
-                handled = Frontend.Session.Command(cd);
-            }
-            if (!handled) {
-                // we may have no network manager yet
-                Engine.IProtocolManager nm = Frontend.FrontendManager.CurrentProtocolManager;
-                if (nm != null) {
-                    handled = nm.Command(cd);
-                } else {
-                    handled = false;
-                }
-            }
-            if (!handled) {
-               _CommandUnknown(cd);
+
+            CommandModel cd = new CommandModel(
+                Frontend.FrontendManager,
+                _Notebook.CurrentChatView.ChatModel,
+                (string) Frontend.UserConfig["Interface/Entry/CommandCharacter"],
+                cmd
+            );
+
+            if (_Command(cd)) {
+                return;
             }
+
+            _CommandManager.Execute(cd);
         }
         
         private bool _Command(CommandModel cd)
@@ -490,7 +500,9 @@ namespace Smuxi.Frontend.Gnome
             TextMessagePartModel msgPart;
             
             msgPart = new TextMessagePartModel();
-            msgPart.Text = _("[Frontend Commands]");
+            // TRANSLATOR: this line is used as a label / category for a
+            // list of commands below
+            msgPart.Text = "[" + _("Frontend Commands") + "]";
             msgPart.Bold = true;
             msg.MessageParts.Add(msgPart);
             
@@ -506,7 +518,10 @@ namespace Smuxi.Frontend.Gnome
             };
             
             foreach (string line in help) { 
-                cd.FrontendManager.AddTextToCurrentChat("-!- " + line);
+                cd.FrontendManager.AddTextToChat(
+                    cd.Chat,
+                    String.Format("-!- {0}", line)
+                );
             }
         }
         
@@ -517,7 +532,10 @@ namespace Smuxi.Frontend.Gnome
 
         private void _CommandEcho(CommandModel cd)
         {
-            cd.FrontendManager.AddTextToCurrentChat("-!- "+cd.Parameter);
+            cd.FrontendManager.AddTextToChat(
+                cd.Chat,
+                String.Format("-!- {0}", cd.Parameter)
+            );
         }
         
         private void _CommandExec(CommandModel cd)
@@ -537,7 +555,7 @@ namespace Smuxi.Frontend.Gnome
                 try {
                     process.Start();
                     output = process.StandardOutput.ReadToEnd();
-                    cd.FrontendManager.AddTextToCurrentChat(output);
+                    cd.FrontendManager.AddTextToChat(cd.Chat, output);
                 } catch {
                 }
             }
@@ -597,17 +615,15 @@ namespace Smuxi.Frontend.Gnome
         {
             _Notebook.CurrentChatView.Clear();
         }
-        
-        private void _CommandUnknown(CommandModel cd)
-        {
-            cd.FrontendManager.AddTextToCurrentChat("-!- " +
-                                String.Format(_(
-                                              "Unknown Command: {0}"),
-                                              cd.Command));
-        }
-        
+
         private void _NickCompletion()
         {
+            // return if we don't support the current ChatView
+            if (!(_Notebook.CurrentChatView is GroupChatView) &&
+                !(_Notebook.CurrentChatView is PersonChatView)) {
+                return;
+            }
+
             int position = CursorPosition;
             string text = Text;
             string word;
@@ -643,34 +659,63 @@ namespace Smuxi.Frontend.Gnome
                 return;
             }
 
+            bool leadingAt = false;
+            // remove leading @ character
+            if (word.StartsWith("@")) {
+                word = word.Substring(1);
+                leadingAt = true;
+            }
+
             // find the possible nickname
             bool found = false;
             bool partial_found = false;
             string nick = null;
-            //GroupChatModel cp = (GroupChatModel) Frontend.FrontendManager.CurrentChat;
-            GroupChatModel cp = (GroupChatModel) _Notebook.CurrentChatView.ChatModel;
-            if ((bool)Frontend.UserConfig["Interface/Entry/BashStyleCompletion"]) {
-                IList<string> result = cp.PersonLookupAll(word);
-                if (result == null || result.Count == 0) {
-                    // no match
-                } else if (result.Count == 1) {
-                    found = true;
-                    nick = result[0];
-                } else if (result.Count >= 2) {
-                    string[] nickArray = new string[result.Count];
-                    result.CopyTo(nickArray, 0);
-                    string nicks = String.Join(" ", nickArray, 1, nickArray.Length - 1);
-                    Frontend.FrontendManager.AddTextToCurrentChat("-!- " + nicks);
-                    found = true;
-                    partial_found = true;
-                    nick = result[0];
+
+            ChatModel chat = _Notebook.CurrentChatView.ChatModel;
+            if (chat.ChatType == ChatType.Group) {
+                GroupChatModel cp = (GroupChatModel) chat;
+                if ((bool)Frontend.UserConfig["Interface/Entry/BashStyleCompletion"]) {
+                    IList<string> result = cp.PersonLookupAll(word);
+                    if (result == null || result.Count == 0) {
+                        // no match
+                    } else if (result.Count == 1) {
+                        found = true;
+                        nick = result[0];
+                    } else if (result.Count >= 2) {
+                        string[] nickArray = new string[result.Count];
+                        result.CopyTo(nickArray, 0);
+                        string nicks = String.Join(" ", nickArray, 1, nickArray.Length - 1);
+                        _Notebook.CurrentChatView.AddMessage(
+                            new MessageModel(String.Format("-!- {0}", nicks))
+                        );
+                        found = true;
+                        partial_found = true;
+                        nick = result[0];
+                    }
+                } else {
+                    PersonModel person = cp.PersonLookup(word);
+                    if (person != null) {
+                        found = true;
+                        nick = person.IdentityName;
+                     }
                 }
             } else {
-                PersonModel person = cp.PersonLookup(word);
-                if (person != null) {
+                PersonChatModel cp = (PersonChatModel) chat;
+                PersonModel person = cp.Person;
+
+                if (person.IdentityName.StartsWith(word, StringComparison.InvariantCultureIgnoreCase)) {
                     found = true;
-                    nick = person.IdentityName;
-                 }
+                    nick = cp.Person.IdentityName;
+                }
+            }
+
+            string completionChar = (string)
+                Frontend.UserConfig["Interface/Entry/CompletionCharacter"];
+            // add leading @ back and supress completion character
+            if (leadingAt) {
+                word = String.Format("@{0}", word);
+                nick = String.Format("@{0}", nick);
+                completionChar = String.Empty;
             }
 
             if (found) {
@@ -692,7 +737,7 @@ namespace Smuxi.Frontend.Gnome
                     if (partial_found) {
                         Text = temp;
                     } else {
-                        Text = temp+" ";
+                        Text = temp + " ";
                     }
                     Position = previous_space + 2 + nick.Length;
                 } else if (next_space != -1) {
@@ -702,15 +747,16 @@ namespace Smuxi.Frontend.Gnome
                         Text = nick + " " + temp;
                         Position = nick.Length;
                     } else {
-                        Text = nick+(string)Frontend.UserConfig["Interface/Entry/CompletionCharacter"] + " " + temp;
-                        Position = nick.Length + 2;
+                        Text = String.Format("{0}{1} {2}", nick,
+                                             completionChar, temp);
+                        Position = nick.Length + completionChar.Length + 1;
                     }
                 } else {
                     // no spaces
                     if (partial_found) {
                         Text = nick;
                     } else {
-                        Text = nick+(string)Frontend.UserConfig["Interface/Entry/CompletionCharacter"]+" ";
+                        Text = String.Format("{0}{1} ", nick, completionChar);
                     }
                     Position = -1;
                 }
@@ -724,53 +770,38 @@ namespace Smuxi.Frontend.Gnome
             if (config == null) {
                 throw new ArgumentNullException("config");
             }
-            
-            string bgStr = (string) config["Interface/Chat/BackgroundColor"];
-            if (!String.IsNullOrEmpty(bgStr)) {
-                Gdk.Color bgColor = Gdk.Color.Zero;
-                if (Gdk.Color.Parse(bgStr, ref bgColor)) {
-                    ModifyBase(Gtk.StateType.Normal, bgColor);
-                }
-            } else {
+
+            var theme = new ThemeSettings(config);
+            if (theme.BackgroundColor == null) {
                 ModifyBase(Gtk.StateType.Normal);
-            }
-            
-            string fgStr = (string) config["Interface/Chat/ForegroundColor"];
-            if (!String.IsNullOrEmpty(fgStr)) {
-                Gdk.Color fgColor = Gdk.Color.Zero;
-                if (Gdk.Color.Parse(fgStr, ref fgColor)) {
-                    ModifyText(Gtk.StateType.Normal, fgColor);
-                }
             } else {
+                ModifyBase(Gtk.StateType.Normal, theme.BackgroundColor.Value);
+            }
+            if (theme.ForegroundColor == null) {
                 ModifyText(Gtk.StateType.Normal);
-            }
-            
-            string fontFamily = (string) config["Interface/Chat/FontFamily"];
-            string fontStyle = (string) config["Interface/Chat/FontStyle"];
-            int fontSize = 0;
-            if (config["Interface/Chat/FontSize"] != null) {
-                fontSize = (int) config["Interface/Chat/FontSize"];
-            }
-            Pango.FontDescription fontDescription = new Pango.FontDescription();
-            if (String.IsNullOrEmpty(fontFamily)) {
-                // use Monospace and Bold by default
-                fontDescription.Family = "monospace";
-                //fontDescription.Weight = Pango.Weight.Bold;
             } else {
-                fontDescription.Family = fontFamily;
-                string frontWeigth = null;
-                if (fontStyle.Contains(" ")) {
-                    int pos = fontStyle.IndexOf(" ");
-                    frontWeigth = fontStyle.Substring(0, pos);
-                    fontStyle = fontStyle.Substring(pos + 1);
-                }
-                fontDescription.Style = (Pango.Style) Enum.Parse(typeof(Pango.Style), fontStyle);
-                if (frontWeigth != null) {
-                    fontDescription.Weight = (Pango.Weight) Enum.Parse(typeof(Pango.Weight), frontWeigth);
-                }
-                fontDescription.Size = fontSize * 1024;
+                ModifyText(Gtk.StateType.Normal, theme.ForegroundColor.Value);
+            }
+            ModifyFont(theme.FontDescription);
+        }
+
+        private void InitCommandManager()
+        {
+            if (_CommandManager != null) {
+                _CommandManager.Dispose();
+            }
+
+            if (Frontend.Session == null) {
+                _CommandManager = null;
+            } else {
+                _CommandManager = new CommandManager(Frontend.Session);
+                _CommandManager.ExceptionEvent +=
+                delegate(object sender, CommandExceptionEventArgs e) {
+                    Gtk.Application.Invoke(delegate {
+                        Frontend.ShowException(e.Exception);
+                    });
+                };
             }
-            ModifyFont(fontDescription);
         }
 
         private static string _(string msg)
