@@ -28,6 +28,7 @@
 
 using System;
 using System.IO;
+using System.Threading;
 using System.Reflection;
 using Smuxi.Engine;
 using Smuxi.Common;
@@ -57,8 +58,11 @@ namespace Smuxi.Frontend.Gnome
         private static Session            _Session;
         private static UserConfig         _UserConfig;
         private static FrontendManager    _FrontendManager;
+        private static TaskQueue          _FrontendManagerCheckerQueue;
         private static object             _UnhandledExceptionSyncRoot = new Object();
-        
+
+        public static event EventHandler  SessionPropertyChanged;
+
         public static string Name {
             get {
                 return _Name;
@@ -118,6 +122,10 @@ namespace Smuxi.Frontend.Gnome
             }
             set {
                 _Session = value;
+
+                if (SessionPropertyChanged != null) {
+                    SessionPropertyChanged(value, EventArgs.Empty);
+                }
             }
         }
         
@@ -261,7 +269,7 @@ namespace Smuxi.Frontend.Gnome
                                                    "local");
             }
             _EngineVersion = Engine.Engine.Version;
-            _Session = _LocalSession;
+            Session = _LocalSession;
             _UserConfig = _Session.UserConfig;
         }
         
@@ -291,26 +299,69 @@ namespace Smuxi.Frontend.Gnome
 
             // local sessions can't have network issues :)
             if (_Session != _LocalSession) {
-                // check once per minute the status of the frontend manager
-                GLib.Timeout.Add(60 * 1000, _CheckFrontendManagerStatus);
+                _FrontendManagerCheckerQueue = new TaskQueue("FrontendManagerCheckerQueue");
+                _FrontendManagerCheckerQueue.AbortedEvent += delegate {
+#if LOG4NET
+                    _Logger.Debug("_FrontendManagerCheckerQueue.AbortedEvent(): task queue aborted!");
+#endif
+                };
+
+                _FrontendManagerCheckerQueue.ExceptionEvent +=
+                delegate(object sender, TaskQueueExceptionEventArgs e) {
+#if LOG4NET
+                    _Logger.Error("Exception in TaskQueue: ", e.Exception);
+                    _Logger.Error("Inner-Exception: ", e.Exception.InnerException);
+#endif
+                    Frontend.ShowException(e.Exception);
+                };
+
+                _FrontendManagerCheckerQueue.Queue(delegate {
+                    // keep looping as long as the checker returns true
+                    while (CheckFrontendManagerStatus()) {
+                        // only check once per minute
+                        Thread.Sleep(60 * 1000);
+                    }
+#if LOG4NET
+                    _Logger.Debug("_FrontendManagerCheckerQueue(): " +
+                                  "CheckFrontendManagerStatus() returned false, "+
+                                  "time to say good bye!");
+#endif
+                });
             }
         }
         
         public static void DisconnectEngineFromGUI()
         {
             Trace.Call();
-            
-            _FrontendManager.IsFrontendDisconnecting = true;
-            _Session.DeregisterFrontendUI(_MainWindow.UI);
+
+            try {
+                _FrontendManager.IsFrontendDisconnecting = true;
+                _Session.DeregisterFrontendUI(_MainWindow.UI);
+            } catch (System.Net.Sockets.SocketException ex) {
+                // ignore as the connection is maybe already broken
+            } catch (System.Runtime.Remoting.RemotingException ex) {
+                // ignore as the connection is maybe already broken
+            }
             _MainWindow.Hide();
             _MainWindow.Notebook.RemoveAllPages();
             // make sure no stray SSH tunnel leaves behind
             _MainWindow.EngineManager.Disconnect();
             
             _FrontendManager = null;
-            _Session = null;
+            Session = null;
         }
-        
+
+        public static void ReconnectEngineToGUI()
+        {
+            Trace.Call();
+
+            Frontend.DisconnectEngineFromGUI();
+            _MainWindow.EngineManager.Reconnect();
+            Session = _MainWindow.EngineManager.Session;
+            _UserConfig = _MainWindow.EngineManager.UserConfig;
+            Frontend.ConnectEngineToGUI();
+        }
+
         public static void Quit()
         {
             Trace.Call();
@@ -334,13 +385,9 @@ namespace Smuxi.Frontend.Gnome
                 _FrontendConfig[Frontend.UIName + "/Interface/YPosition"] = y;
                 _FrontendConfig.Save();
             }
-            
+
             if (_FrontendManager != null) {
-                try {
-                    DisconnectEngineFromGUI();
-                } catch (System.Runtime.Remoting.RemotingException ex) {
-                    // the connection is maybe already gone
-                }
+                DisconnectEngineFromGUI();
             }
             
             Gtk.Application.Quit();
@@ -407,17 +454,46 @@ namespace Smuxi.Frontend.Gnome
 #if LOG4NET
             _Logger.Error("ShowException(): Exception:", ex);
 #endif
-            
-            if (ex is System.Runtime.Remoting.RemotingException) {
+
+            // HACK: ugly MS .NET throws underlaying SocketException instead of
+            // wrapping those into a nice RemotingException, see:
+            // http://projects.qnetp.net/issues/show/232
+            if (ex is System.Runtime.Remoting.RemotingException ||
+                ex is System.Net.Sockets.SocketException) {
                 Gtk.MessageDialog md = new Gtk.MessageDialog(parent,
                     Gtk.DialogFlags.Modal, Gtk.MessageType.Error,
-                    Gtk.ButtonsType.OkCancel, _("The frontend has lost the connection to the server.\n Do you want to reconnect now?"));
+                    Gtk.ButtonsType.OkCancel, _("The frontend has lost the connection to the server.\nDo you want to reconnect now?"));
                 Gtk.ResponseType res = (Gtk.ResponseType) md.Run();
                 md.Destroy();
                 
                 if (res == Gtk.ResponseType.Ok) {
-                    Frontend.DisconnectEngineFromGUI();
-                    _MainWindow.EngineManager.Reconnect();
+                    while (true) {
+                        try {
+                            Frontend.ReconnectEngineToGUI();
+                            // yay, we made it
+                            break;
+                        } catch (Exception e) {
+#if LOG4NET
+                             _Logger.Error("ShowException(): Reconnect failed, exception:", e);
+#endif
+                            var msg = _("Reconnecting to the server has failed.\nDo you want to try again?");
+                            // the parent window is hidden (MainWindow) at this
+                            // point thus modal doesn't make sense here
+                            md = new Gtk.MessageDialog(parent,
+                                Gtk.DialogFlags.DestroyWithParent,
+                                Gtk.MessageType.Error,
+                                Gtk.ButtonsType.OkCancel, msg);
+                            md.SetPosition(Gtk.WindowPosition.CenterAlways);
+                            res = (Gtk.ResponseType) md.Run();
+                            md.Destroy();
+
+                            if (res != Gtk.ResponseType.Ok) {
+                                // give up
+                                Quit();
+                                return;
+                            }
+                        }
+                    }
                     return;
                 }
                 
@@ -529,38 +605,38 @@ namespace Smuxi.Frontend.Gnome
         }
 #endif
         
-        private static bool _CheckFrontendManagerStatus()
+        private static bool CheckFrontendManagerStatus()
         {
             Trace.Call();
+
+            if (_FrontendManager == null) {
+                // we lost the frontend manager, nothing to check
+                return false;
+            }
+
+            if (_FrontendManager.IsAlive) {
+                // everything is fine
+                return true;
+            }
             
-            try {
-                if (_FrontendManager == null) {
-                    // we lost the frontend manager, nothing to check
-                    return false;
-                }
-                
-                if (_FrontendManager.IsAlive) {
-                    return true;
-                }
-                
 #if LOG4NET
-                _Logger.Error("_CheckFrontendManagerStatus(): frontend manager is not alive anymore!");
+            _Logger.Error("CheckFrontendManagerStatus(): frontend manager is not alive anymore!");
 #endif
+            Gtk.Application.Invoke(delegate {
                 Gtk.MessageDialog md = new Gtk.MessageDialog(_MainWindow,
                     Gtk.DialogFlags.Modal, Gtk.MessageType.Error,
-                    Gtk.ButtonsType.OkCancel, _("The server has lost the connection to the frontend.\n Do you want to reconnect now?"));
+                    Gtk.ButtonsType.OkCancel, _("The server has lost the connection to the frontend.\nDo you want to reconnect now?"));
                 Gtk.ResponseType res = (Gtk.ResponseType) md.Run();
                 md.Destroy();
                 
                 if (res != Gtk.ResponseType.Ok) {
-                    return false;
+                    // the frontend is unusable in this state -> say good bye
+                    Frontend.Quit();
+                    return;
                 }
-                
-                Frontend.DisconnectEngineFromGUI();
-                _MainWindow.EngineManager.Reconnect();
-            } catch (Exception ex) {
-                Frontend.ShowException(ex);
-            }
+
+                Frontend.ReconnectEngineToGUI();
+            });
 
             return false;
         }
