@@ -47,7 +47,6 @@ namespace Smuxi.Frontend.Gnome
 #endif
         private int _Version = 0;
         private ChatViewManager _ChatViewManager;
-        private IList<ChatView> _SyncedChatViews;
         
         public int Version {
             get {
@@ -55,12 +54,6 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
-        public IList<ChatView> SyncedChatViews {
-            get {
-                return _SyncedChatViews;
-            }
-        }
-        
         public GnomeUI(ChatViewManager chatViewManager)
         {
             _ChatViewManager = chatViewManager;
@@ -78,16 +71,35 @@ namespace Smuxi.Frontend.Gnome
             });
         }
         
-        private void _AddMessageToChat(ChatModel epage, MessageModel msg)
+        private void _AddMessageToChat(ChatModel chatModel, MessageModel msg)
         {
-            ChatView chatView = _ChatViewManager.GetChat(epage);
-#if LOG4NET
+            ChatView chatView = _ChatViewManager.GetChat(chatModel);
             if (chatView == null) {
-                _Logger.Fatal(String.Format("_AddMessageToChat(): Notebook.GetPage(epage) epage.Name: {0} returned null!", epage.Name));
+#if LOG4NET
+                _Logger.Fatal(
+                    String.Format(
+                        "_AddMessageToChat(): " +
+                        "_ChatViewManager.GetChat(chatModel) " +
+                        "chatModel.Name: {0} returned null!",
+                        chatModel.Name
+                    )
+                );
+#endif
                 return;
             }
-#endif
+            
+            DateTime start, stop;
+            start = DateTime.UtcNow;
             chatView.AddMessage(msg);
+            stop = DateTime.UtcNow;
+#if LOG4NET
+            _Logger.Debug(
+                String.Format(
+                    "_AddMessageToChat(): chatView.AddMessage() took: {0:0.00} ms",
+                    (stop - start).TotalMilliseconds
+                )
+            );
+#endif
         }
         
         public void AddMessageToChat(ChatModel epage, MessageModel fmsg)
@@ -147,6 +159,20 @@ namespace Smuxi.Frontend.Gnome
                 TraceRemotingCall(mb, chatModel);
 
                 ChatView chatView = _ChatViewManager.GetChat(chatModel);
+                if (chatView == null) {
+#if LOG4NET
+                    _Logger.Fatal(
+                        String.Format(
+                            "SyncChat(): " +
+                            "_ChatViewManager.GetChat(chatModel) " +
+                            "chatModel.Name: {0} returned null!",
+                            chatModel.Name
+                        )
+                    );
+#endif
+                    return;
+                }
+
 #if LOG4NET
                 DateTime syncStart = DateTime.UtcNow;
 #endif
@@ -156,10 +182,10 @@ namespace Smuxi.Frontend.Gnome
                 double duration = syncStop.Subtract(syncStart).TotalMilliseconds;
                 _Logger.Debug("SyncChat() done, syncing took: " + Math.Round(duration) + " ms");
 #endif
-                
+
                 // maybe a BUG here? should be tell the FrontendManager before we sync?
                 Frontend.FrontendManager.AddSyncedChat(chatModel);
-                //_SyncedChats.Add(chatView);
+
                 // BUG: doesn't work?!?
                 chatView.ScrollToEnd();
                 
@@ -183,6 +209,19 @@ namespace Smuxi.Frontend.Gnome
                 TraceRemotingCall(mb, groupChat, person);
                 
                 GroupChatView groupChatView = (GroupChatView) _ChatViewManager.GetChat(groupChat);
+                if (groupChatView == null) {
+#if LOG4NET
+                    _Logger.Fatal(
+                        String.Format(
+                            "AddPersonToGroupChat(): " +
+                            "_ChatViewManager.GetChat(groupChat) " +
+                            "groupChat.Name: {0} returned null!",
+                            groupChat.Name
+                        )
+                    );
+#endif
+                    return;
+                }
                 groupChatView.AddPerson(person);
             });
         }
@@ -196,20 +235,46 @@ namespace Smuxi.Frontend.Gnome
                 TraceRemotingCall(mb, groupChat, oldPerson, newPerson);
                 
                 GroupChatView groupChatView = (GroupChatView) _ChatViewManager.GetChat(groupChat);
+                if (groupChatView == null) {
+#if LOG4NET
+                    _Logger.Fatal(
+                        String.Format(
+                            "UpdatePersonInGroupChat(): " +
+                            "_ChatViewManager.GetChat(groupChat) " +
+                            "groupChat.Name: {0} returned null!",
+                            groupChat.Name
+                        )
+                    );
+#endif
+                    return;
+                }
                 groupChatView.UpdatePerson(oldPerson, newPerson);
             });
         }
         
-        public void UpdateTopicInGroupChat(GroupChatModel ecpage, MessageModel topic)
+        public void UpdateTopicInGroupChat(GroupChatModel groupChat, MessageModel topic)
         {
-            TraceRemotingCall(ecpage, topic);
+            TraceRemotingCall(groupChat, topic);
 
             MethodBase mb = Trace.GetMethodBase();
             Gtk.Application.Invoke(delegate {
-                TraceRemotingCall(mb, ecpage, topic);
+                TraceRemotingCall(mb, groupChat, topic);
                 
-                GroupChatView cpage = (GroupChatView)Frontend.MainWindow.Notebook.GetChat(ecpage);
-                cpage.SetTopic(topic);
+                GroupChatView groupChatView = (GroupChatView)Frontend.MainWindow.Notebook.GetChat(groupChat);
+                if (groupChatView == null) {
+#if LOG4NET
+                    _Logger.Fatal(
+                        String.Format(
+                            "UpdateTopicInGroupChat(): " +
+                            "_ChatViewManager.GetChat(groupChat) " +
+                            "groupChat.Name: {0} returned null!",
+                            groupChat.Name
+                        )
+                    );
+#endif
+                    return;
+                }
+                groupChatView.SetTopic(topic);
             });
         }
         
@@ -222,6 +287,19 @@ namespace Smuxi.Frontend.Gnome
                 TraceRemotingCall(mb, groupChat, person);
             
                 GroupChatView groupChatView = (GroupChatView) _ChatViewManager.GetChat(groupChat);
+                if (groupChatView == null) {
+#if LOG4NET
+                    _Logger.Fatal(
+                        String.Format(
+                            "RemovePersonFromGroupChat(): " +
+                            "_ChatViewManager.GetChat(groupChat) " +
+                            "groupChat.Name: {0} returned null!",
+                            groupChat.Name
+                        )
+                    );
+#endif
+                    return;
+                }
                 groupChatView.RemovePerson(person);
             });
         }
