@@ -28,6 +28,7 @@
 
 using System;
 using System.IO;
+using System.Threading;
 using System.Reflection;
 using Mono.Unix;
 using Smuxi.Common;
@@ -120,7 +121,7 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
-        public MainWindow() : base("Smuxi - Smart MUtipleXed Irc")
+        public MainWindow() : base("Smuxi")
         {
             // restore window size / position
             int width, heigth;
@@ -373,11 +374,16 @@ namespace Smuxi.Frontend.Gnome
             
             _Statusbar = new Gtk.Statusbar();
             
-            Gtk.HBox sb_hbox = new Gtk.HBox();
-            sb_hbox.PackStart(_NetworkStatusbar, false, true, 0);
-            sb_hbox.PackStart(_Statusbar, true, true, 0);
-            sb_hbox.PackStart(_ProgressBar, false, false, 0);
-            vbox.PackStart(sb_hbox, false, false, 0);
+            Gtk.HBox status_bar_hbox = new Gtk.HBox();
+            status_bar_hbox.Homogeneous = true;
+            status_bar_hbox.PackStart(_NetworkStatusbar, false, true, 0);
+            status_bar_hbox.PackStart(_Statusbar, true, true, 0);
+
+            Gtk.HBox status_hbox = new Gtk.HBox();
+            status_hbox.PackStart(status_bar_hbox);
+            status_hbox.PackStart(_ProgressBar, false, false, 0);
+
+            vbox.PackStart(status_hbox, false, false, 0);
             Add(vbox);
         }
 
@@ -442,6 +448,23 @@ namespace Smuxi.Frontend.Gnome
                 // HACK: out of scope?
                 Frontend.StatusIcon.Blinking = false;
 #endif
+                ChatView chatView = _Notebook.CurrentChatView;
+                if (chatView != null) {
+                    // clear activity and highlight
+                    chatView.HasHighlight = false;
+                    chatView.HasActivity = false;
+                    // update last seen highlight
+                    ThreadPool.QueueUserWorkItem(delegate {
+                        try {
+                            // REMOTING CALL 1
+                            chatView.ChatModel.LastSeenHighlight = DateTime.UtcNow;
+                        } catch (Exception ex) {
+#if LOG4NET
+                            f_Logger.Error("OnFocusInEvent(): Exception", ex);
+#endif
+                        }
+                    });
+                }
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
@@ -467,14 +490,7 @@ namespace Smuxi.Frontend.Gnome
 #endif
                 }
                 
-                Frontend.Session.Connect(
-                        Frontend.FrontendManager,
-                        server.Protocol,
-                        server.Hostname,
-                        server.Port,
-                        server.Username,
-                        server.Password
-                );
+                Frontend.Session.Connect(server, Frontend.FrontendManager);
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
@@ -487,17 +503,18 @@ namespace Smuxi.Frontend.Gnome
             
             try {
                 ServerListController controller = new ServerListController(Frontend.UserConfig);
-                ServerView serverView = new ServerView(this,
+                ServerDialog dialog = new ServerDialog(this,
                                                        null,
                                                        Frontend.Session.GetSupportedProtocols(),
                                                        controller.GetNetworks());
-                int res = serverView.Run();
-                serverView.Destroy();
+                int res = dialog.Run();
+                ServerModel server = dialog.GetServer();
+                dialog.Destroy();
                 if (res != (int) Gtk.ResponseType.Ok) {
                     return;
                 }
                 
-                controller.AddServer(serverView.Server);
+                controller.AddServer(server);
                 controller.Save();
             } catch (InvalidOperationException ex) {
                 Frontend.ShowError(this, _("Unable to add server: "), ex);
@@ -864,10 +881,6 @@ namespace Smuxi.Frontend.Gnome
 #if GTK_SHARP_2_10
                 Frontend.StatusIcon.Blinking = true;
 #endif
-                if (Frontend.UserConfig["Sound/BeepOnHighlight"] != null &&
-                    (bool)Frontend.UserConfig["Sound/BeepOnHighlight"]) {
-                    Display.Beep();
-                }
             }
         }
 
