@@ -7,7 +7,7 @@
  *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2008 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2010 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -36,7 +36,7 @@ using Smuxi.Frontend;
 namespace Smuxi.Frontend.Gnome
 {
     // TODO: use Gtk.Bin
-    public abstract class ChatView : Gtk.EventBox, IChatView
+    public abstract class ChatView : Gtk.EventBox, IChatView, IDisposable
     {
 #if LOG4NET
         private static readonly log4net.ILog _Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
@@ -46,6 +46,7 @@ namespace Smuxi.Frontend.Gnome
         private   bool               _HasHighlight;
         private   bool               _HasActivity;
         private   bool               _HasEvent;
+        private   bool               _IsSynced;
         private   Gtk.TextMark       _EndMark;
         private   Gtk.Menu           _TabMenu;
         private   Gtk.Label          _TabLabel;
@@ -54,6 +55,8 @@ namespace Smuxi.Frontend.Gnome
         private   Gtk.ScrolledWindow _OutputScrolledWindow;
         private   MessageTextView    _OutputMessageTextView;
         private   ThemeSettings      _ThemeSettings;
+        private   DateTime           _LastHighlight;
+        private   TaskQueue          _LastSeenHighlightQueue;
         
         public ChatModel ChatModel {
             get {
@@ -73,34 +76,53 @@ namespace Smuxi.Frontend.Gnome
                     HasActivity = false;
                     return;
                 }
-                
-                string color = (string) Frontend.UserConfig["Interface/Notebook/Tab/HighlightColor"];
-                _TabLabel.Markup = String.Format("<span foreground=\"{0}\">{1}</span>", color, _Name);
+
+                var color = ColorTools.GetBestTextColor(
+                    ColorTools.GetTextColor(_ThemeSettings.HighlightColor),
+                    ColorTools.GetTextColor(
+                        Gtk.Rc.GetStyle(_TabLabel).Base(Gtk.StateType.Normal)
+                    ), ColorContrast.High
+                );
+                _TabLabel.Markup = String.Format(
+                    "<span foreground=\"{0}\">{1}</span>",
+                    GLib.Markup.EscapeText(color.ToString()),
+                    GLib.Markup.EscapeText(_Name)
+                );
             }
         }
-        
+
         public bool HasActivity {
             get {
                 return _HasActivity;
             }
             set {
                 _HasActivity = value;
-                
+
                 if (HasHighlight) {
                     // don't show activity if there is a highlight active
                     return;
                 }
-                
-                string color = null;
+
+                Gdk.Color colorValue;
                 if (value) {
-                    color = (string) Frontend.UserConfig["Interface/Notebook/Tab/ActivityColor"];
+                    colorValue = _ThemeSettings.ActivityColor;
                 } else {
-                    color = (string) Frontend.UserConfig["Interface/Notebook/Tab/NoActivityColor"];
+                    colorValue = _ThemeSettings.NoActivityColor;
                 }
-                _TabLabel.Markup = String.Format("<span foreground=\"{0}\">{1}</span>", color, _Name);
+                var color = ColorTools.GetBestTextColor(
+                    ColorTools.GetTextColor(colorValue),
+                    ColorTools.GetTextColor(
+                        Gtk.Rc.GetStyle(_TabLabel).Base(Gtk.StateType.Normal)
+                    ), ColorContrast.High
+                );
+                _TabLabel.Markup = String.Format(
+                    "<span foreground=\"{0}\">{1}</span>",
+                    GLib.Markup.EscapeText(color.ToString()),
+                    GLib.Markup.EscapeText(_Name)
+                );
             }
         }
-        
+
         public bool HasEvent {
             get {
                 return _HasEvent;
@@ -118,9 +140,18 @@ namespace Smuxi.Frontend.Gnome
                     HasActivity = false;
                     return;
                 }
-                
-                string color = (string) Frontend.UserConfig["Interface/Notebook/Tab/EventColor"];
-                _TabLabel.Markup = String.Format("<span foreground=\"{0}\">{1}</span>", color, _Name);
+
+                var color = ColorTools.GetBestTextColor(
+                    ColorTools.GetTextColor(_ThemeSettings.EventColor),
+                    ColorTools.GetTextColor(
+                        Gtk.Rc.GetStyle(_TabLabel).Base(Gtk.StateType.Normal)
+                    ), ColorContrast.High
+                );
+                _TabLabel.Markup = String.Format(
+                    "<span foreground=\"{0}\">{1}</span>",
+                    GLib.Markup.EscapeText(color.ToString()),
+                    GLib.Markup.EscapeText(_Name)
+                );
             }
         }
         
@@ -144,7 +175,7 @@ namespace Smuxi.Frontend.Gnome
                 return _TabEventBox;
             }
         }
-        
+
         public MessageTextView OutputMessageTextView {
             get {
                 return _OutputMessageTextView;
@@ -183,7 +214,6 @@ namespace Smuxi.Frontend.Gnome
             tv.Editable = false;
             tv.CursorVisible = true;
             tv.WrapMode = Gtk.WrapMode.Char;
-            tv.Buffer.Changed += new EventHandler(_OnTextBufferChanged);
             tv.MessageAdded += OnMessageTextViewMessageAdded;
             tv.MessageHighlighted += OnMessageTextViewMessageHighlighted;
             _OutputMessageTextView = tv;
@@ -221,8 +251,42 @@ namespace Smuxi.Frontend.Gnome
             _TabEventBox.ShowAll();
 
             _ThemeSettings = new ThemeSettings();
+
+            // OPT-TODO: this should use a TaskStack instead of TaskQueue
+            _LastSeenHighlightQueue = new TaskQueue("LastSeenHighlightQueue("+_Name+")");
+            _LastSeenHighlightQueue.AbortedEvent += OnLastSeenHighlightQueueAbortedEvent;
+            _LastSeenHighlightQueue.ExceptionEvent += OnLastSeenHighlightQueueExceptionEvent;
         }
         
+        ~ChatView()
+        {
+            Trace.Call();
+
+            Dispose(false);
+        }
+
+        public override void Dispose()
+        {
+            Trace.Call();
+
+            base.Dispose();
+
+            Dispose(true);
+            GC.SuppressFinalize(this);
+        }
+
+        protected void Dispose(bool disposing)
+        {
+            Trace.Call(disposing);
+
+            if (disposing) {
+                if (_LastSeenHighlightQueue != null) {
+                    _LastSeenHighlightQueue.Dispose();
+                }
+                _LastSeenHighlightQueue = null;
+            }
+        }
+
         public virtual void ScrollUp()
         {
             Trace.Call();
@@ -322,6 +386,12 @@ namespace Smuxi.Frontend.Gnome
                     AddMessage(msg);
                 }
             }
+            // REMOTING CALL
+            if (_LastHighlight > _ChatModel.LastSeenHighlight) {
+                HasHighlight = true;
+            }
+
+            _IsSynced = true;
         }
         
         public virtual void AddMessage(MessageModel msg)
@@ -347,42 +417,20 @@ namespace Smuxi.Frontend.Gnome
             }
             
             _ThemeSettings = new ThemeSettings(config);
-            
+
             _OutputMessageTextView.ApplyConfig(config);
         }
         
         public virtual void Close()
         {
             Trace.Call();
-        }
-                        
-        private void _OnTextBufferChanged(object sender, EventArgs e)
-        {
-            Trace.Call(sender, e);
-        
-            Gtk.ScrolledWindow sw = _OutputScrolledWindow;
-            Gtk.TextView tv = _OutputMessageTextView;
-            
-            if (sw.Vadjustment.Upper == (sw.Vadjustment.Value + sw.Vadjustment.PageSize)) {
-                // the scrollbar is way at the end, lets autoscroll
-                Gtk.TextIter endit = tv.Buffer.EndIter;
-                tv.Buffer.PlaceCursor(endit);
-                tv.Buffer.MoveMark(tv.Buffer.InsertMark, endit);
-                tv.ScrollMarkOnscreen(tv.Buffer.InsertMark);
-            }
-            
-            int buffer_lines = (int)Frontend.UserConfig["Interface/Notebook/BufferLines"];
-            if (tv.Buffer.LineCount > buffer_lines) {
-                Gtk.TextIter start_iter = tv.Buffer.StartIter; 
-                // TODO: maybe we should delete chunks instead of each line
-                Gtk.TextIter end_iter = tv.Buffer.GetIterAtLine(tv.Buffer.LineCount - buffer_lines);
-                tv.Buffer.Delete(ref start_iter, ref end_iter);
-            }
 
-            // update the end mark
-            tv.Buffer.MoveMark(_EndMark, tv.Buffer.EndIter);
+            ChatModel.ProtocolManager.CloseChat(
+                Frontend.FrontendManager,
+                ChatModel
+            );
         }
-        
+
         protected virtual void OnTabButtonPress(object sender, Gtk.ButtonPressEventArgs e)
         {
             Trace.Call(sender, e);
@@ -408,18 +456,38 @@ namespace Smuxi.Frontend.Gnome
             
             // HACK: out of scope?
             // probably we should use the ChatViewManager instead?
-            if (Frontend.MainWindow.Notebook.CurrentChatView == this) {
-                return;
+            if (Frontend.MainWindow.Notebook.CurrentChatView != this) {
+                switch (e.Message.MessageType) {
+                    case MessageType.Normal:
+                        HasActivity = true;
+                        break;
+                    case MessageType.Event:
+                        HasEvent = true;
+                        break;
+                }
             }
-            
-            switch (e.Message.MessageType) {
-                case MessageType.Normal:
-                    HasActivity = true;
-                    break;
-                case MessageType.Event:
-                    HasEvent = true;
-                    break;
+
+            Gtk.ScrolledWindow sw = _OutputScrolledWindow;
+            Gtk.TextView tv = _OutputMessageTextView;
+
+            if (sw.Vadjustment.Upper == (sw.Vadjustment.Value + sw.Vadjustment.PageSize)) {
+                // the scrollbar is way at the end, lets autoscroll
+                Gtk.TextIter endit = tv.Buffer.EndIter;
+                tv.Buffer.PlaceCursor(endit);
+                tv.Buffer.MoveMark(tv.Buffer.InsertMark, endit);
+                tv.ScrollMarkOnscreen(tv.Buffer.InsertMark);
             }
+
+            int buffer_lines = (int)Frontend.UserConfig["Interface/Notebook/BufferLines"];
+            if (tv.Buffer.LineCount > buffer_lines) {
+                Gtk.TextIter start_iter = tv.Buffer.StartIter;
+                // TODO: maybe we should delete chunks instead of each line
+                Gtk.TextIter end_iter = tv.Buffer.GetIterAtLine(tv.Buffer.LineCount - buffer_lines);
+                tv.Buffer.Delete(ref start_iter, ref end_iter);
+            }
+
+            // update the end mark
+            tv.Buffer.MoveMark(_EndMark, tv.Buffer.EndIter);
         }
         
         protected virtual void OnMessageTextViewMessageHighlighted(object sender, MessageTextViewMessageHighlightedEventArgs e)
@@ -427,15 +495,78 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
             
             // HACK: out of scope?
-            if (Frontend.MainWindow.Notebook.CurrentChatView == this) {
-                return;
+            // only beep if the main windows has no focus (the user is
+            // elsewhere) and the chat is was already synced, as during sync we
+            // would get insane from all beeping caused by the old highlights
+            if (!Frontend.MainWindow.HasToplevelFocus &&
+                _IsSynced &&
+                Frontend.UserConfig["Sound/BeepOnHighlight"] != null &&
+                (bool) Frontend.UserConfig["Sound/BeepOnHighlight"]) {
+#if LOG4NET
+                _Logger.Debug("OnMessageTextViewMessageHighlighted(): BEEP!");
+#endif
+                Display.Beep();
             }
-            
-            if (_ChatModel.LastSeenHighlight < e.Message.TimeStamp) {
-                HasHighlight = true;
+
+            if (_IsSynced) {
+                bool isActiveChat =
+                    Frontend.MainWindow.HasToplevelFocus &&
+                    Object.ReferenceEquals(
+                        Frontend.MainWindow.Notebook.CurrentChatView,
+                        this
+                    );
+
+                var method = Trace.GetMethodBase();
+                // update last seen highlight
+                // OPT-TODO: we should use a TaskStack here OR at least a
+                // timeout approach that will only sync once per 30 seconds!
+                _LastSeenHighlightQueue.Queue(delegate {
+                    Trace.Call(method, null, null);
+
+                    // unhandled exception here would kill the syncer thread
+                    try {
+                        if (isActiveChat) {
+                            // REMOTING CALL 1
+                            _ChatModel.LastSeenHighlight = e.Message.TimeStamp;
+                        } else {
+                            // REMOTING CALL 1
+                            if (_ChatModel.LastSeenHighlight < e.Message.TimeStamp) {
+                                Gtk.Application.Invoke(delegate {
+                                    HasHighlight = true;
+                                });
+                            }
+                        }
+                    } catch (Exception ex) {
+#if LOG4NET
+                        _Logger.Error("OnMessageTextViewMessageHighlighted(): Exception: ", ex);
+#endif
+                    }
+                });
+            } else {
+                _LastHighlight = e.Message.TimeStamp;
             }
         }
-        
+
+        protected virtual void OnLastSeenHighlightQueueExceptionEvent(object sender, TaskQueueExceptionEventArgs e)
+        {
+            Trace.Call(sender, e);
+
+#if LOG4NET
+            _Logger.Error("Exception in TaskQueue: ", e.Exception);
+            _Logger.Error("Inner-Exception: ", e.Exception.InnerException);
+#endif
+            Frontend.ShowException(e.Exception);
+        }
+
+        protected virtual void OnLastSeenHighlightQueueAbortedEvent(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+
+#if LOG4NET
+            _Logger.Debug("OnLastSeenHighlightQueueAbortedEvent(): task queue aborted!");
+#endif
+        }
+
         private static string _(string msg)
         {
             return Mono.Unix.Catalog.GetString(msg);
