@@ -68,13 +68,17 @@ namespace Smuxi.Frontend.Gnome
 
         public override bool HasSelection {
             get {
-                return base.HasSelection || _PersonTreeView.Selection.CountSelectedRows() > 0;
+                return base.HasSelection ||
+                       _PersonTreeView.Selection.CountSelectedRows() > 0 ||
+                       _TopicTextView.HasTextViewSelection;
             }
         }
-        
+
         public override bool HasFocus {
             get {
-                return base.HasFocus || _PersonTreeView.HasFocus;
+                return base.HasFocus ||
+                       _PersonTreeView.HasFocus ||
+                       _TopicTextView.HasFocus;
             }
         }
         
@@ -125,7 +129,7 @@ namespace Smuxi.Frontend.Gnome
             column.Spacing = 0;
             column.SortIndicator = false;
             column.Sizing = Gtk.TreeViewColumnSizing.Autosize;
-            column.SetCellDataFunc(cellr, new Gtk.TreeCellDataFunc(_RenderPersonIdentityName));
+            column.SetCellDataFunc(cellr, new Gtk.TreeCellDataFunc(RenderPersonIdentityName));
             tv.AppendColumn(column);
             _IdentityNameColumn = column;
             
@@ -145,6 +149,7 @@ namespace Smuxi.Frontend.Gnome
             _PersonMenu.TakeFocus = false;
             
             _PersonTreeView.ButtonPressEvent += _OnPersonTreeViewButtonPressEvent;
+            _PersonTreeView.KeyPressEvent += OnPersonTreeViewKeyPressEvent;
             // frame needed for events when selecting something in the treeview
             _PersonTreeViewFrame = new Gtk.Frame();
             _PersonTreeViewFrame.ButtonReleaseEvent += new Gtk.ButtonReleaseEventHandler(_OnUserListButtonReleaseEvent);
@@ -163,7 +168,12 @@ namespace Smuxi.Frontend.Gnome
             _TopicScrolledWindow.HscrollbarPolicy = Gtk.PolicyType.Automatic;
             _TopicScrolledWindow.VscrollbarPolicy = Gtk.PolicyType.Automatic;
             _TopicScrolledWindow.Add(_TopicTextView);
-            
+            // make sure the topic is invisible and remains by default and
+            // visible when a topic gets set
+            _TopicScrolledWindow.ShowAll();
+            _TopicScrolledWindow.Visible = false;
+            _TopicScrolledWindow.NoShowAll = true;
+
             // predict and set useful topic heigth
             Pango.Layout layout = _TopicTextView.CreatePangoLayout("Test Topic");
             int lineWidth, lineHeigth;
@@ -207,7 +217,10 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call();
 
-            IDictionary<string, PersonModel> persons = _GroupChatModel.Persons; 
+            IDictionary<string, PersonModel> persons = _GroupChatModel.Persons;
+            if (persons == null) {
+                persons = new Dictionary<string, PersonModel>(0);
+            }
 #if LOG4NET
             _Logger.Debug("Sync() syncing persons");
 #endif
@@ -225,7 +238,7 @@ namespace Smuxi.Frontend.Gnome
                 
                 // HACK: out of scope
                 string status = String.Format(
-                                    _("Syncing chat persons of {0}..."),
+                                    _("Retrieving user list for {0}..."),
                                     ChatModel.Name);
                 Frontend.MainWindow.Statusbar.Push(0, status);
     
@@ -269,10 +282,11 @@ namespace Smuxi.Frontend.Gnome
                
                 // HACK: out of scope
                 Frontend.MainWindow.ProgressBar.Fraction = 0;
-                status += _(" done.");
+                // TRANSLATOR: this string will be appended to the one above
+                status += String.Format(" {0}", _("done."));
                 Frontend.MainWindow.Statusbar.Push(0, status);
             }
-           
+
 #if LOG4NET
             _Logger.Debug("Sync() syncing topic");
 #endif
@@ -370,7 +384,8 @@ namespace Smuxi.Frontend.Gnome
 
             _TopicTextView.Clear();
             _TopicTextView.AddMessage(topic, false);
-                                     
+            _TopicScrolledWindow.Visible = true;
+
             /*
             foreach (MessagePartModel topicPart in topic.MessageParts) {
 #if LOG4NET
@@ -508,9 +523,9 @@ namespace Smuxi.Frontend.Gnome
             _OutputHPaned.ShowAll();
         }
 
-        private void _RenderPersonIdentityName(Gtk.TreeViewColumn column,
-                                               Gtk.CellRenderer cellr,
-                                               Gtk.TreeModel model, Gtk.TreeIter iter)
+        public virtual void RenderPersonIdentityName(Gtk.TreeViewColumn column,
+                                                     Gtk.CellRenderer cellr,
+                                                     Gtk.TreeModel model, Gtk.TreeIter iter)
         {
             PersonModel person = (PersonModel) model.GetValue(iter, 0);
             (cellr as Gtk.CellRendererText).Text = person.IdentityName;
@@ -532,8 +547,29 @@ namespace Smuxi.Frontend.Gnome
         protected virtual void OnPersonsRowActivated(object sender, Gtk.RowActivatedArgs e)
         {
             Trace.Call(sender, e);
+
+            IList<PersonModel> persons = GetSelectedPersons();
+            if (persons == null) {
+                return;
+            }
+
+            // this is a generic implemention that should be able to open/create
+            // a private chat in most cases, as it depends what OpenChat()
+            // of the specific protocol actually expects/needs
+            foreach (PersonModel person in persons) {
+                PersonChatModel personChat = new PersonChatModel(
+                    person,
+                    person.ID,
+                    person.IdentityName,
+                    ChatModel.ProtocolManager
+                );
+                ChatModel.ProtocolManager.OpenChat(
+                    Frontend.FrontendManager,
+                    personChat
+                );
+            }
         }
-        
+
         protected virtual void OnPersonTreeViewFocusOutEvent(object sender, EventArgs e)
         {
             Trace.Call(sender, e);
@@ -542,12 +578,35 @@ namespace Smuxi.Frontend.Gnome
             _PersonTreeView.Selection.UnselectAll();
         }
         
+        protected virtual void OnPersonTreeViewKeyPressEvent(object sender, Gtk.KeyPressEventArgs e)
+        {
+            Trace.Call(sender, e);
+
+            if ((e.Event.State & Gdk.ModifierType.Mod1Mask) != 0 ||
+                (e.Event.State & Gdk.ModifierType.ControlMask) != 0 ||
+                (e.Event.State & Gdk.ModifierType.ShiftMask) != 0) {
+                // alt, ctrl or shift pushed, returning
+                return;
+            }
+
+            if (e.Event.Key == Gdk.Key.Menu &&
+                _PersonTreeView.Selection.CountSelectedRows() > 0) {
+                _PersonMenu.Popup(null, null, null, 0, e.Event.Time);
+                _PersonMenu.ShowAll();
+            }
+        }
+
         private void _OnUserListButtonReleaseEvent(object sender, Gtk.ButtonReleaseEventArgs e)
         {
             Trace.Call(sender, e);
 
             if (e.Event.Button == 3 && _PersonTreeView.Selection.CountSelectedRows() > 0) {
-                _PersonMenu.Popup(null, null, null, e.Event.Button, e.Event.Time);
+                // HACK: don't pass the real mouse button that was used to
+                // initiate the menu, as sub-menus will only respond to that
+                // button for some reason! As workaround we always pass
+                // 0 == left mouse button here
+                //_PersonMenu.Popup(null, null, null, e.Event.Button, e.Event.Time);
+                _PersonMenu.Popup(null, null, null, 0, e.Event.Time);
                 _PersonMenu.ShowAll();
             }
         }
