@@ -45,6 +45,7 @@ namespace Smuxi.Frontend.Gnome
         private bool         _ShowTimestamps;
         private bool         _ShowHighlight;
         private bool         _AtUrlTag;
+        private string       _Url;
         private UserConfig   _Config;
         private ThemeSettings _ThemeSettings;
 
@@ -96,6 +97,7 @@ namespace Smuxi.Frontend.Gnome
             
             Buffer = new Gtk.TextBuffer(_MessageTextTagTable);
             MotionNotifyEvent += OnMotionNotifyEvent;
+            PopulatePopup += OnPopulatePopup;
         }
 
         public void ApplyConfig(UserConfig config)
@@ -115,7 +117,13 @@ namespace Smuxi.Frontend.Gnome
             
             string wrapModeStr = (string) config["Interface/Chat/WrapMode"];
             if (!String.IsNullOrEmpty(wrapModeStr)) {
-                Gtk.WrapMode wrapMode = (Gtk.WrapMode) Enum.Parse(typeof(Gtk.WrapMode), wrapModeStr);
+                Gtk.WrapMode wrapMode = (Gtk.WrapMode) Enum.Parse(
+                    typeof(Gtk.WrapMode),
+                    wrapModeStr
+                );
+                if (wrapMode == Gtk.WrapMode.Word) {
+                    wrapMode = Gtk.WrapMode.WordChar;
+                }
                 WrapMode = wrapMode;
             }
         }
@@ -139,7 +147,13 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(msg, addLinebreak);
             
             Gtk.TextIter iter = Buffer.EndIter;
-            
+
+            Gdk.Color bgColor = DefaultAttributes.Appearance.BgColor;
+            if (_ThemeSettings.BackgroundColor != null) {
+                bgColor = _ThemeSettings.BackgroundColor.Value;
+            }
+            TextColor bgTextColor = ColorTools.GetTextColor(bgColor);
+
             if (_ShowTimestamps) {
                 DateTime localTimestamp = msg.TimeStamp.ToLocalTime();
                 if (_LastMessage != null &&
@@ -162,25 +176,33 @@ namespace Smuxi.Frontend.Gnome
                 }
 
                 if (timestamp != null) {
-                    Buffer.Insert(ref iter, timestamp + " ");
+                    timestamp = String.Format("{0} ", timestamp);
+                    if (msg.MessageType == MessageType.Event) {
+                        // get best contrast for the event font color
+                        Gtk.TextTag eventTag = _MessageTextTagTable.Lookup("event");
+                        Gdk.Color eventColor = eventTag.ForegroundGdk;
+                        TextColor eventTextColor = ColorTools.GetBestTextColor(
+                            ColorTools.GetTextColor(eventColor),
+                            bgTextColor,
+                            ColorContrast.High
+                        );
+                        eventTag.ForegroundGdk = ColorTools.GetGdkColor(
+                            eventTextColor
+                        );
+                        Buffer.InsertWithTagsByName(ref iter, timestamp, "event");
+                    } else {
+                        Buffer.Insert(ref iter, timestamp);
+                    }
                 }
             }
 
             bool hasHighlight = false;
             foreach (MessagePartModel msgPart in msg.MessageParts) {
-#if LOG4NET
-                _Logger.Debug("AddMessage(): msgPart.GetType(): " + msgPart.GetType());
-#endif
                 // supposed to be used only in a ChatView
                 if (msgPart.IsHighlight) {
                     hasHighlight = true;
                 }
-                
-                Gdk.Color bgColor = DefaultAttributes.Appearance.BgColor;
-                if (_ThemeSettings.BackgroundColor != null) {
-                    bgColor = _ThemeSettings.BackgroundColor.Value;
-                }
-                TextColor bgTextColor = ColorTools.GetTextColor(bgColor);
+
                 // TODO: implement all types
                 if (msgPart is UrlMessagePartModel) {
                     UrlMessagePartModel fmsgui = (UrlMessagePartModel) msgPart;
@@ -195,9 +217,6 @@ namespace Smuxi.Frontend.Gnome
                     Buffer.InsertWithTagsByName(ref iter, fmsgui.Url, "url");
                 } else if (msgPart is TextMessagePartModel) {
                     TextMessagePartModel fmsgti = (TextMessagePartModel) msgPart;
-#if LOG4NET
-                    _Logger.Debug("AddMessage(): fmsgti.Text: '" + fmsgti.Text + "'");
-#endif
                     List<string> tags = new List<string>();
                     if (fmsgti.ForegroundColor != TextColor.None) {
                         TextColor color = ColorTools.GetBestTextColor(fmsgti.ForegroundColor, bgTextColor);
@@ -229,9 +248,18 @@ namespace Smuxi.Frontend.Gnome
 #endif
                         tags.Add("italic");
                     }
-                    
-                    Buffer.InsertWithTagsByName(ref iter, fmsgti.Text, tags.ToArray());
-                } 
+                    if (msg.MessageType == MessageType.Event &&
+                        fmsgti.ForegroundColor == TextColor.None) {
+                        // only mark parts that don't have a color set
+                        tags.Add("event");
+                    }
+
+                    if (tags.Count > 0) {
+                        Buffer.InsertWithTagsByName(ref iter, fmsgti.Text, tags.ToArray());
+                    } else {
+                        Buffer.Insert(ref iter, fmsgti.Text);
+                    }
+                }
             }
             if (addLinebreak) {
                 Buffer.Insert(ref iter, "\n");
@@ -242,11 +270,11 @@ namespace Smuxi.Frontend.Gnome
             // for local messages. See:
             // http://projects.qnetp.net/issues/show/185
             QueueDraw();
-            
+
             if (MessageAdded != null) {
                 MessageAdded(this, new MessageTextViewMessageAddedEventArgs(msg));
             }
-            
+
             if (hasHighlight) {
                 if (MessageHighlighted != null) {
                     MessageHighlighted(this, new MessageTextViewMessageHighlightedEventArgs(msg));
@@ -289,7 +317,11 @@ namespace Smuxi.Frontend.Gnome
             fd = new Pango.FontDescription();
             tt.FontDesc = fd;
             ttt.Add(tt);
-            
+
+            tt = new Gtk.TextTag("event");
+            tt.Foreground = "darkgray";
+            ttt.Add(tt);
+
             return ttt;
         }
 
@@ -336,11 +368,8 @@ namespace Smuxi.Frontend.Gnome
         
         protected virtual void OnTextTagUrlTextEvent(object sender, Gtk.TextEventArgs e)
         {
-            Trace.Call(sender, e);
-            
-            if (e.Event.Type != Gdk.EventType.ButtonRelease) {
-                return;
-            }
+            // logging noise
+            //Trace.Call(sender, e);
             
             Gtk.TextIter start = Gtk.TextIter.Zero;
             Gtk.TextIter end = Gtk.TextIter.Zero;
@@ -360,29 +389,74 @@ namespace Smuxi.Frontend.Gnome
             start.BackwardToTagToggle(tag);
             end = e.Iter;
             end.ForwardToTagToggle(tag);
-            string url = Buffer.GetText(start, end, false);
-            if (String.IsNullOrEmpty(url)) {
+            _Url = Buffer.GetText(start, end, false);
+
+            if (e.Event.Type != Gdk.EventType.ButtonRelease) {
+                return;
+            }
+
+            if (String.IsNullOrEmpty(_Url)) {
 #if LOG4NET
                 _Logger.Warn("OnTextTagUrlTextEvent(): url is empty, ignoring...");
 #endif
                 return;
             }
-            
-            if (!Regex.IsMatch(url, @"^[a-zA-Z0-9\-]+:\/\/")) {
+
+            OpenLink(_Url);
+        }
+
+        protected virtual void OnPopulatePopup(object sender, Gtk.PopulatePopupArgs e)
+        {
+            Trace.Call(sender, e);
+
+            if (!_AtUrlTag) {
+                return;
+            }
+
+            Gtk.Menu popup = e.Menu;
+            // remove all items
+            foreach (Gtk.Widget children in popup.Children) {
+                popup.Remove(children);
+            }
+
+            Gtk.ImageMenuItem open_item = new Gtk.ImageMenuItem(Gtk.Stock.Open, null);
+            open_item.Activated += delegate {
+                if (!String.IsNullOrEmpty(_Url)) {
+                    OpenLink(_Url);
+                }
+            };
+            popup.Append(open_item);
+
+            Gtk.ImageMenuItem copy_item = new Gtk.ImageMenuItem(Gtk.Stock.Copy, null);
+            copy_item.Activated += delegate {
+                Gdk.Atom clipboardAtom = Gdk.Atom.Intern("CLIPBOARD", false);
+                Gtk.Clipboard clipboard = Gtk.Clipboard.Get(clipboardAtom);
+                clipboard.Text = _Url;
+            };
+            popup.Append(copy_item);
+
+            popup.ShowAll();
+        }
+
+        private void OpenLink(string link)
+        {
+            Trace.Call(link);
+
+            if (!Regex.IsMatch(link, @"^[a-zA-Z0-9\-]+:\/\/")) {
                 // URL doesn't start with a protocol
-                url = "http://" + url;
+                link = "http://" + link;
             }
             
             // hopefully MS .NET / Mono finds some way to handle the URL
             ThreadPool.QueueUserWorkItem(delegate {
                 try {
-                    SysDiag.Process.Start(url);
+                    SysDiag.Process.Start(link);
                 } catch (Exception ex) {
                     // exceptions in the thread pool would kill the process, see:
                     // http://msdn.microsoft.com/en-us/library/0ka9477y.aspx
                     // http://projects.qnetp.net/issues/show/194
 #if LOG4NET
-                    _Logger.Error("OnTextTagUrlTextEvent(): opening URL failed", ex);
+                    _Logger.Error("OpenLink(): opening URL: '" + link + "' failed", ex);
 #endif
                 }
             });
