@@ -1,11 +1,11 @@
 /*
- * $Id: AssemblyInfo.cs 324 2008-07-24 23:26:34Z zhila $
- * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-Test/AssemblyInfo.cs $
- * $Rev: 324 $
- * $Author: zhila $
- * $Date: 2008-07-25 01:26:34 +0200 (Fri, 25 Jul 2008) $
+ * $Id$
+ * $URL$
+ * $Rev$
+ * $Author$
+ * $Date$
  *
- * smuxi - Smart MUltipleXed Irc
+ * Smuxi - Smart MUltipleXed Irc
  *
  * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
  *
@@ -29,7 +29,7 @@
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
-[assembly: AssemblyTitle("smuxi - Test frontend")]
+[assembly: AssemblyTitle("Smuxi - Test frontend")]
 [assembly: AssemblyCopyright("2005-2008 (C) Mirco Bauer <meebey@meebey.net>")]
 
 [assembly: AssemblyDelaySign(false)]
